@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,53 +13,65 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import abc
-import typing
+from typing import Awaitable, Callable, Dict, Optional, Sequence, Union
+import packaging.version
 import pkg_resources
 
-from google import auth  # type: ignore
-from google.api_core import exceptions  # type: ignore
+import google.auth  # type: ignore
+import google.api_core  # type: ignore
+from google.api_core import exceptions as core_exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
 from google.api_core import operations_v1  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 
 from google.cloud.asset_v1.types import asset_service
-from google.longrunning import operations_pb2 as operations  # type: ignore
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.longrunning import operations_pb2  # type: ignore
+from google.protobuf import empty_pb2  # type: ignore
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
-        gapic_version=pkg_resources.get_distribution(
-            "google-cloud-asset",
-        ).version,
+        gapic_version=pkg_resources.get_distribution("google-cloud-asset",).version,
     )
 except pkg_resources.DistributionNotFound:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()
 
+try:
+    # google.auth.__version__ was added in 1.26.0
+    _GOOGLE_AUTH_VERSION = google.auth.__version__
+except AttributeError:
+    try:  # try pkg_resources if it is available
+        _GOOGLE_AUTH_VERSION = pkg_resources.get_distribution("google-auth").version
+    except pkg_resources.DistributionNotFound:  # pragma: NO COVER
+        _GOOGLE_AUTH_VERSION = None
+
+_API_CORE_VERSION = google.api_core.__version__
+
 
 class AssetServiceTransport(abc.ABC):
     """Abstract transport class for AssetService."""
 
     AUTH_SCOPES = ("https://www.googleapis.com/auth/cloud-platform",)
 
+    DEFAULT_HOST: str = "cloudasset.googleapis.com"
+
     def __init__(
         self,
         *,
-        host: str = "cloudasset.googleapis.com",
-        credentials: credentials.Credentials = None,
-        credentials_file: typing.Optional[str] = None,
-        scopes: typing.Optional[typing.Sequence[str]] = AUTH_SCOPES,
-        quota_project_id: typing.Optional[str] = None,
+        host: str = DEFAULT_HOST,
+        credentials: ga_credentials.Credentials = None,
+        credentials_file: Optional[str] = None,
+        scopes: Optional[Sequence[str]] = None,
+        quota_project_id: Optional[str] = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
         **kwargs,
     ) -> None:
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -69,7 +80,7 @@ class AssetServiceTransport(abc.ABC):
             credentials_file (Optional[str]): A file with credentials that can
                 be loaded with :func:`google.auth.load_credentials_from_file`.
                 This argument is mutually exclusive with credentials.
-            scope (Optional[Sequence[str]]): A list of scopes.
+            scopes (Optional[Sequence[str]]): A list of scopes.
             quota_project_id (Optional[str]): An optional project to use for billing
                 and quota.
             client_info (google.api_core.gapic_v1.client_info.ClientInfo):
@@ -83,34 +94,94 @@ class AssetServiceTransport(abc.ABC):
             host += ":443"
         self._host = host
 
+        scopes_kwargs = self._get_scopes_kwargs(self._host, scopes)
+
+        # Save the scopes.
+        self._scopes = scopes or self.AUTH_SCOPES
+
         # If no credentials are provided, then determine the appropriate
         # defaults.
         if credentials and credentials_file:
-            raise exceptions.DuplicateCredentialArgs(
+            raise core_exceptions.DuplicateCredentialArgs(
                 "'credentials_file' and 'credentials' are mutually exclusive"
             )
 
         if credentials_file is not None:
-            credentials, _ = auth.load_credentials_from_file(
-                credentials_file, scopes=scopes, quota_project_id=quota_project_id
+            credentials, _ = google.auth.load_credentials_from_file(
+                credentials_file, **scopes_kwargs, quota_project_id=quota_project_id
             )
 
         elif credentials is None:
-            credentials, _ = auth.default(
-                scopes=scopes, quota_project_id=quota_project_id
+            credentials, _ = google.auth.default(
+                **scopes_kwargs, quota_project_id=quota_project_id
             )
 
         # Save the credentials.
         self._credentials = credentials
 
-        # Lifted into its own function so it can be stubbed out during tests.
-        self._prep_wrapped_messages(client_info)
+    # TODO(busunkim): These two class methods are in the base transport
+    # to avoid duplicating code across the transport classes. These functions
+    # should be deleted once the minimum required versions of google-api-core
+    # and google-auth are increased.
+
+    # TODO: Remove this function once google-auth >= 1.25.0 is required
+    @classmethod
+    def _get_scopes_kwargs(
+        cls, host: str, scopes: Optional[Sequence[str]]
+    ) -> Dict[str, Optional[Sequence[str]]]:
+        """Returns scopes kwargs to pass to google-auth methods depending on the google-auth version"""
+
+        scopes_kwargs = {}
+
+        if _GOOGLE_AUTH_VERSION and (
+            packaging.version.parse(_GOOGLE_AUTH_VERSION)
+            >= packaging.version.parse("1.25.0")
+        ):
+            scopes_kwargs = {"scopes": scopes, "default_scopes": cls.AUTH_SCOPES}
+        else:
+            scopes_kwargs = {"scopes": scopes or cls.AUTH_SCOPES}
+
+        return scopes_kwargs
+
+    # TODO: Remove this function once google-api-core >= 1.26.0 is required
+    @classmethod
+    def _get_self_signed_jwt_kwargs(
+        cls, host: str, scopes: Optional[Sequence[str]]
+    ) -> Dict[str, Union[Optional[Sequence[str]], str]]:
+        """Returns kwargs to pass to grpc_helpers.create_channel depending on the google-api-core version"""
+
+        self_signed_jwt_kwargs: Dict[str, Union[Optional[Sequence[str]], str]] = {}
+
+        if _API_CORE_VERSION and (
+            packaging.version.parse(_API_CORE_VERSION)
+            >= packaging.version.parse("1.26.0")
+        ):
+            self_signed_jwt_kwargs["default_scopes"] = cls.AUTH_SCOPES
+            self_signed_jwt_kwargs["scopes"] = scopes
+            self_signed_jwt_kwargs["default_host"] = cls.DEFAULT_HOST
+        else:
+            self_signed_jwt_kwargs["scopes"] = scopes or cls.AUTH_SCOPES
+
+        return self_signed_jwt_kwargs
 
     def _prep_wrapped_messages(self, client_info):
         # Precompute the wrapped methods.
         self._wrapped_methods = {
             self.export_assets: gapic_v1.method.wrap_method(
-                self.export_assets,
+                self.export_assets, default_timeout=60.0, client_info=client_info,
+            ),
+            self.list_assets: gapic_v1.method.wrap_method(
+                self.list_assets,
+                default_retry=retries.Retry(
+                    initial=0.1,
+                    maximum=60.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
                 default_timeout=60.0,
                 client_info=client_info,
             ),
@@ -121,17 +192,16 @@ class AssetServiceTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
+                    deadline=60.0,
                 ),
                 default_timeout=60.0,
                 client_info=client_info,
             ),
             self.create_feed: gapic_v1.method.wrap_method(
-                self.create_feed,
-                default_timeout=60.0,
-                client_info=client_info,
+                self.create_feed, default_timeout=60.0, client_info=client_info,
             ),
             self.get_feed: gapic_v1.method.wrap_method(
                 self.get_feed,
@@ -140,9 +210,10 @@ class AssetServiceTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
+                    deadline=60.0,
                 ),
                 default_timeout=60.0,
                 client_info=client_info,
@@ -154,17 +225,16 @@ class AssetServiceTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
+                    deadline=60.0,
                 ),
                 default_timeout=60.0,
                 client_info=client_info,
             ),
             self.update_feed: gapic_v1.method.wrap_method(
-                self.update_feed,
-                default_timeout=60.0,
-                client_info=client_info,
+                self.update_feed, default_timeout=60.0, client_info=client_info,
             ),
             self.delete_feed: gapic_v1.method.wrap_method(
                 self.delete_feed,
@@ -173,9 +243,10 @@ class AssetServiceTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
+                    deadline=60.0,
                 ),
                 default_timeout=60.0,
                 client_info=client_info,
@@ -187,9 +258,10 @@ class AssetServiceTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
+                    deadline=15.0,
                 ),
                 default_timeout=15.0,
                 client_info=client_info,
@@ -201,9 +273,10 @@ class AssetServiceTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.ServiceUnavailable,
                     ),
+                    deadline=15.0,
                 ),
                 default_timeout=15.0,
                 client_info=client_info,
@@ -215,8 +288,9 @@ class AssetServiceTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.ServiceUnavailable,
                     ),
+                    deadline=300.0,
                 ),
                 default_timeout=300.0,
                 client_info=client_info,
@@ -236,20 +310,32 @@ class AssetServiceTransport(abc.ABC):
     @property
     def export_assets(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.ExportAssetsRequest],
-        typing.Union[operations.Operation, typing.Awaitable[operations.Operation]],
+        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
+    ]:
+        raise NotImplementedError()
+
+    @property
+    def list_assets(
+        self,
+    ) -> Callable[
+        [asset_service.ListAssetsRequest],
+        Union[
+            asset_service.ListAssetsResponse,
+            Awaitable[asset_service.ListAssetsResponse],
+        ],
     ]:
         raise NotImplementedError()
 
     @property
     def batch_get_assets_history(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.BatchGetAssetsHistoryRequest],
-        typing.Union[
+        Union[
             asset_service.BatchGetAssetsHistoryResponse,
-            typing.Awaitable[asset_service.BatchGetAssetsHistoryResponse],
+            Awaitable[asset_service.BatchGetAssetsHistoryResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -257,29 +343,28 @@ class AssetServiceTransport(abc.ABC):
     @property
     def create_feed(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.CreateFeedRequest],
-        typing.Union[asset_service.Feed, typing.Awaitable[asset_service.Feed]],
+        Union[asset_service.Feed, Awaitable[asset_service.Feed]],
     ]:
         raise NotImplementedError()
 
     @property
     def get_feed(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.GetFeedRequest],
-        typing.Union[asset_service.Feed, typing.Awaitable[asset_service.Feed]],
+        Union[asset_service.Feed, Awaitable[asset_service.Feed]],
     ]:
         raise NotImplementedError()
 
     @property
     def list_feeds(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.ListFeedsRequest],
-        typing.Union[
-            asset_service.ListFeedsResponse,
-            typing.Awaitable[asset_service.ListFeedsResponse],
+        Union[
+            asset_service.ListFeedsResponse, Awaitable[asset_service.ListFeedsResponse]
         ],
     ]:
         raise NotImplementedError()
@@ -287,29 +372,29 @@ class AssetServiceTransport(abc.ABC):
     @property
     def update_feed(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.UpdateFeedRequest],
-        typing.Union[asset_service.Feed, typing.Awaitable[asset_service.Feed]],
+        Union[asset_service.Feed, Awaitable[asset_service.Feed]],
     ]:
         raise NotImplementedError()
 
     @property
     def delete_feed(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.DeleteFeedRequest],
-        typing.Union[empty.Empty, typing.Awaitable[empty.Empty]],
+        Union[empty_pb2.Empty, Awaitable[empty_pb2.Empty]],
     ]:
         raise NotImplementedError()
 
     @property
     def search_all_resources(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.SearchAllResourcesRequest],
-        typing.Union[
+        Union[
             asset_service.SearchAllResourcesResponse,
-            typing.Awaitable[asset_service.SearchAllResourcesResponse],
+            Awaitable[asset_service.SearchAllResourcesResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -317,11 +402,11 @@ class AssetServiceTransport(abc.ABC):
     @property
     def search_all_iam_policies(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.SearchAllIamPoliciesRequest],
-        typing.Union[
+        Union[
             asset_service.SearchAllIamPoliciesResponse,
-            typing.Awaitable[asset_service.SearchAllIamPoliciesResponse],
+            Awaitable[asset_service.SearchAllIamPoliciesResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -329,11 +414,11 @@ class AssetServiceTransport(abc.ABC):
     @property
     def analyze_iam_policy(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.AnalyzeIamPolicyRequest],
-        typing.Union[
+        Union[
             asset_service.AnalyzeIamPolicyResponse,
-            typing.Awaitable[asset_service.AnalyzeIamPolicyResponse],
+            Awaitable[asset_service.AnalyzeIamPolicyResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -341,9 +426,9 @@ class AssetServiceTransport(abc.ABC):
     @property
     def analyze_iam_policy_longrunning(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [asset_service.AnalyzeIamPolicyLongrunningRequest],
-        typing.Union[operations.Operation, typing.Awaitable[operations.Operation]],
+        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
     ]:
         raise NotImplementedError()
 
