@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,19 +13,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import proto  # type: ignore
 
-
-from google.cloud.orgpolicy.v1 import orgpolicy_pb2 as orgpolicy  # type: ignore
+from google.cloud.orgpolicy.v1 import orgpolicy_pb2  # type: ignore
 from google.cloud.osconfig_v1 import Inventory  # type: ignore
-from google.iam.v1 import policy_pb2 as gi_policy  # type: ignore
-from google.identity.accesscontextmanager.v1 import access_level_pb2 as gia_access_level  # type: ignore
-from google.identity.accesscontextmanager.v1 import access_policy_pb2 as gia_access_policy  # type: ignore
-from google.identity.accesscontextmanager.v1 import service_perimeter_pb2 as gia_service_perimeter  # type: ignore
-from google.protobuf import struct_pb2 as struct  # type: ignore
-from google.protobuf import timestamp_pb2 as timestamp  # type: ignore
-from google.rpc import code_pb2 as gr_code  # type: ignore
+from google.iam.v1 import policy_pb2  # type: ignore
+from google.identity.accesscontextmanager.v1 import access_level_pb2  # type: ignore
+from google.identity.accesscontextmanager.v1 import access_policy_pb2  # type: ignore
+from google.identity.accesscontextmanager.v1 import service_perimeter_pb2  # type: ignore
+from google.protobuf import struct_pb2  # type: ignore
+from google.protobuf import timestamp_pb2  # type: ignore
+from google.rpc import code_pb2  # type: ignore
 
 
 __protobuf__ = proto.module(
@@ -39,6 +36,7 @@ __protobuf__ = proto.module(
         "ResourceSearchResult",
         "IamPolicySearchResult",
         "IamPolicyAnalysisState",
+        "ConditionEvaluation",
         "IamPolicyAnalysisResult",
     },
 )
@@ -50,16 +48,16 @@ class TemporalAsset(proto.Message):
     window.
 
     Attributes:
-        window (~.assets.TimeWindow):
+        window (google.cloud.asset_v1.types.TimeWindow):
             The time window when the asset data and state
             was observed.
         deleted (bool):
             Whether the asset has been deleted or not.
-        asset (~.assets.Asset):
+        asset (google.cloud.asset_v1.types.Asset):
             An asset in Google Cloud.
-        prior_asset_state (~.assets.TemporalAsset.PriorAssetState):
+        prior_asset_state (google.cloud.asset_v1.types.TemporalAsset.PriorAssetState):
             State of prior_asset.
-        prior_asset (~.assets.Asset):
+        prior_asset (google.cloud.asset_v1.types.Asset):
             Prior copy of the asset. Populated if prior_asset_state is
             PRESENT. Currently this is only set for responses in
             Real-Time Feed.
@@ -73,56 +71,26 @@ class TemporalAsset(proto.Message):
         DOES_NOT_EXIST = 3
         DELETED = 4
 
-    window = proto.Field(
-        proto.MESSAGE,
-        number=1,
-        message="TimeWindow",
-    )
-
-    deleted = proto.Field(proto.BOOL, number=2)
-
-    asset = proto.Field(
-        proto.MESSAGE,
-        number=3,
-        message="Asset",
-    )
-
-    prior_asset_state = proto.Field(
-        proto.ENUM,
-        number=4,
-        enum=PriorAssetState,
-    )
-
-    prior_asset = proto.Field(
-        proto.MESSAGE,
-        number=5,
-        message="Asset",
-    )
+    window = proto.Field(proto.MESSAGE, number=1, message="TimeWindow",)
+    deleted = proto.Field(proto.BOOL, number=2,)
+    asset = proto.Field(proto.MESSAGE, number=3, message="Asset",)
+    prior_asset_state = proto.Field(proto.ENUM, number=4, enum=PriorAssetState,)
+    prior_asset = proto.Field(proto.MESSAGE, number=5, message="Asset",)
 
 
 class TimeWindow(proto.Message):
     r"""A time window specified by its ``start_time`` and ``end_time``.
-
     Attributes:
-        start_time (~.timestamp.Timestamp):
+        start_time (google.protobuf.timestamp_pb2.Timestamp):
             Start time of the time window (exclusive).
-        end_time (~.timestamp.Timestamp):
+        end_time (google.protobuf.timestamp_pb2.Timestamp):
             End time of the time window (inclusive). If
             not specified, the current timestamp is used
             instead.
     """
 
-    start_time = proto.Field(
-        proto.MESSAGE,
-        number=1,
-        message=timestamp.Timestamp,
-    )
-
-    end_time = proto.Field(
-        proto.MESSAGE,
-        number=2,
-        message=timestamp.Timestamp,
-    )
+    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,)
+    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)
 
 
 class Asset(proto.Message):
@@ -131,12 +99,13 @@ class Asset(proto.Message):
     hierarchy <https://cloud.google.com/resource-manager/docs/cloud-platform-resource-hierarchy>`__,
     a resource outside the Google Cloud resource hierarchy (such as
     Google Kubernetes Engine clusters and objects), or a policy (e.g.
-    Cloud IAM policy). See `Supported asset
+    Cloud IAM policy), or a relationship (e.g. an
+    INSTANCE_TO_INSTANCEGROUP relationship). See `Supported asset
     types <https://cloud.google.com/asset-inventory/docs/supported-asset-types>`__
     for more information.
 
     Attributes:
-        update_time (~.timestamp.Timestamp):
+        update_time (google.protobuf.timestamp_pb2.Timestamp):
             The last update timestamp of an asset. update_time is
             updated when create/update/delete operation is performed.
         name (str):
@@ -153,9 +122,9 @@ class Asset(proto.Message):
             See `Supported asset
             types <https://cloud.google.com/asset-inventory/docs/supported-asset-types>`__
             for more information.
-        resource (~.assets.Resource):
+        resource (google.cloud.asset_v1.types.Resource):
             A representation of the resource.
-        iam_policy (~.gi_policy.Policy):
+        iam_policy (google.iam.v1.policy_pb2.Policy):
             A representation of the Cloud IAM policy set on a Google
             Cloud resource. There can be a maximum of one Cloud IAM
             policy set on any given resource. In addition, Cloud IAM
@@ -167,21 +136,21 @@ class Asset(proto.Message):
             See `this
             topic <https://cloud.google.com/iam/docs/policies#inheritance>`__
             for more information.
-        org_policy (Sequence[~.orgpolicy.Policy]):
+        org_policy (Sequence[google.cloud.orgpolicy.v1.orgpolicy_pb2.Policy]):
             A representation of an `organization
             policy <https://cloud.google.com/resource-manager/docs/organization-policy/overview#organization_policy>`__.
             There can be more than one organization policy with
             different constraints set on a given resource.
-        access_policy (~.gia_access_policy.AccessPolicy):
+        access_policy (google.identity.accesscontextmanager.v1.access_policy_pb2.AccessPolicy):
             Please also refer to the `access policy user
             guide <https://cloud.google.com/access-context-manager/docs/overview#access-policies>`__.
-        access_level (~.gia_access_level.AccessLevel):
+        access_level (google.identity.accesscontextmanager.v1.access_level_pb2.AccessLevel):
             Please also refer to the `access level user
             guide <https://cloud.google.com/access-context-manager/docs/overview#access-levels>`__.
-        service_perimeter (~.gia_service_perimeter.ServicePerimeter):
+        service_perimeter (google.identity.accesscontextmanager.v1.service_perimeter_pb2.ServicePerimeter):
             Please also refer to the `service perimeter user
             guide <https://cloud.google.com/vpc-service-controls/docs/overview>`__.
-        os_inventory (~.inventory.Inventory):
+        os_inventory (google.cloud.osconfig_v1.Inventory):
             A representation of runtime OS Inventory information. See
             `this
             topic <https://cloud.google.com/compute/docs/instances/os-inventory-management>`__
@@ -200,66 +169,39 @@ class Asset(proto.Message):
     """
 
     update_time = proto.Field(
-        proto.MESSAGE,
-        number=11,
-        message=timestamp.Timestamp,
+        proto.MESSAGE, number=11, message=timestamp_pb2.Timestamp,
     )
-
-    name = proto.Field(proto.STRING, number=1)
-
-    asset_type = proto.Field(proto.STRING, number=2)
-
-    resource = proto.Field(
-        proto.MESSAGE,
-        number=3,
-        message="Resource",
-    )
-
-    iam_policy = proto.Field(
-        proto.MESSAGE,
-        number=4,
-        message=gi_policy.Policy,
-    )
-
+    name = proto.Field(proto.STRING, number=1,)
+    asset_type = proto.Field(proto.STRING, number=2,)
+    resource = proto.Field(proto.MESSAGE, number=3, message="Resource",)
+    iam_policy = proto.Field(proto.MESSAGE, number=4, message=policy_pb2.Policy,)
     org_policy = proto.RepeatedField(
-        proto.MESSAGE,
-        number=6,
-        message=orgpolicy.Policy,
+        proto.MESSAGE, number=6, message=orgpolicy_pb2.Policy,
     )
-
     access_policy = proto.Field(
         proto.MESSAGE,
         number=7,
         oneof="access_context_policy",
-        message=gia_access_policy.AccessPolicy,
+        message=access_policy_pb2.AccessPolicy,
     )
-
     access_level = proto.Field(
         proto.MESSAGE,
         number=8,
         oneof="access_context_policy",
-        message=gia_access_level.AccessLevel,
+        message=access_level_pb2.AccessLevel,
     )
-
     service_perimeter = proto.Field(
         proto.MESSAGE,
         number=9,
         oneof="access_context_policy",
-        message=gia_service_perimeter.ServicePerimeter,
+        message=service_perimeter_pb2.ServicePerimeter,
     )
-
-    os_inventory = proto.Field(
-        proto.MESSAGE,
-        number=12,
-        message=Inventory,
-    )
-
-    ancestors = proto.RepeatedField(proto.STRING, number=10)
+    os_inventory = proto.Field(proto.MESSAGE, number=12, message=Inventory,)
+    ancestors = proto.RepeatedField(proto.STRING, number=10,)
 
 
 class Resource(proto.Message):
     r"""A representation of a Google Cloud resource.
-
     Attributes:
         version (str):
             The API version. Example: ``v1``
@@ -295,7 +237,7 @@ class Resource(proto.Message):
             ``//cloudresourcemanager.googleapis.com/projects/my_project_123``
 
             For third-party assets, this field may be set differently.
-        data (~.struct.Struct):
+        data (google.protobuf.struct_pb2.Struct):
             The content of the resource, in which some
             sensitive fields are removed and may not be
             present.
@@ -306,23 +248,13 @@ class Resource(proto.Message):
             https://cloud.google.com/about/locations/.
     """
 
-    version = proto.Field(proto.STRING, number=1)
-
-    discovery_document_uri = proto.Field(proto.STRING, number=2)
-
-    discovery_name = proto.Field(proto.STRING, number=3)
-
-    resource_url = proto.Field(proto.STRING, number=4)
-
-    parent = proto.Field(proto.STRING, number=5)
-
-    data = proto.Field(
-        proto.MESSAGE,
-        number=6,
-        message=struct.Struct,
-    )
-
-    location = proto.Field(proto.STRING, number=8)
+    version = proto.Field(proto.STRING, number=1,)
+    discovery_document_uri = proto.Field(proto.STRING, number=2,)
+    discovery_name = proto.Field(proto.STRING, number=3,)
+    resource_url = proto.Field(proto.STRING, number=4,)
+    parent = proto.Field(proto.STRING, number=5,)
+    data = proto.Field(proto.MESSAGE, number=6, message=struct_pb2.Struct,)
+    location = proto.Field(proto.STRING, number=8,)
 
 
 class ResourceSearchResult(proto.Message):
@@ -350,14 +282,40 @@ class ResourceSearchResult(proto.Message):
             -  specify the ``asset_type`` field in your search request.
         project (str):
             The project that this resource belongs to, in the form of
-            projects/{PROJECT_NUMBER}.
+            projects/{PROJECT_NUMBER}. This field is available when the
+            resource belongs to a project.
 
-            To search against the ``project``:
+            To search against ``project``:
 
+            -  use a field query. Example: ``project:12345``
+            -  use a free text query. Example: ``12345``
             -  specify the ``scope`` field as this project in your
                search request.
+        folders (Sequence[str]):
+            The folder(s) that this resource belongs to, in the form of
+            folders/{FOLDER_NUMBER}. This field is available when the
+            resource belongs to one or more folders.
+
+            To search against ``folders``:
+
+            -  use a field query. Example: ``folders:(123 OR 456)``
+            -  use a free text query. Example: ``123``
+            -  specify the ``scope`` field as this folder in your search
+               request.
+        organization (str):
+            The organization that this resource belongs to, in the form
+            of organizations/{ORGANIZATION_NUMBER}. This field is
+            available when the resource belongs to an organization.
+
+            To search against ``organization``:
+
+            -  use a field query. Example: ``organization:123``
+            -  use a free text query. Example: ``123``
+            -  specify the ``scope`` field as this organization in your
+               search request.
         display_name (str):
-            The display name of this resource.
+            The display name of this resource. This field is available
+            only when the resource's proto contains it.
 
             To search against the ``display_name``:
 
@@ -365,27 +323,29 @@ class ResourceSearchResult(proto.Message):
             -  use a free text query. Example: ``"My Instance"``
         description (str):
             One or more paragraphs of text description of this resource.
-            Maximum length could be up to 1M bytes.
+            Maximum length could be up to 1M bytes. This field is
+            available only when the resource's proto contains it.
 
             To search against the ``description``:
 
             -  use a field query. Example:
-               ``description:"*important instance*"``
-            -  use a free text query. Example:
-               ``"*important instance*"``
+               ``description:"important instance"``
+            -  use a free text query. Example: ``"important instance"``
         location (str):
             Location can be ``global``, regional like ``us-east1``, or
-            zonal like ``us-west1-b``.
+            zonal like ``us-west1-b``. This field is available only when
+            the resource's proto contains it.
 
             To search against the ``location``:
 
             -  use a field query. Example: ``location:us-west*``
             -  use a free text query. Example: ``us-west*``
-        labels (Sequence[~.assets.ResourceSearchResult.LabelsEntry]):
+        labels (Sequence[google.cloud.asset_v1.types.ResourceSearchResult.LabelsEntry]):
             Labels associated with this resource. See `Labelling and
             grouping GCP
             resources <https://cloud.google.com/blog/products/gcp/labelling-and-grouping-your-google-cloud-platform-resources>`__
-            for more information.
+            for more information. This field is available only when the
+            resource's proto contains it.
 
             To search against the ``labels``:
 
@@ -403,13 +363,79 @@ class ResourceSearchResult(proto.Message):
             network tags are a type of annotations used to group GCP
             resources. See `Labelling GCP
             resources <https://cloud.google.com/blog/products/gcp/labelling-and-grouping-your-google-cloud-platform-resources>`__
-            for more information.
+            for more information. This field is available only when the
+            resource's proto contains it.
 
             To search against the ``network_tags``:
 
             -  use a field query. Example: ``networkTags:internal``
             -  use a free text query. Example: ``internal``
-        additional_attributes (~.struct.Struct):
+        kms_key (str):
+            The Cloud KMS
+            `CryptoKey <https://cloud.google.com/kms/docs/reference/rest/v1/projects.locations.keyRings.cryptoKeys?hl=en>`__
+            name or
+            `CryptoKeyVersion <https://cloud.google.com/kms/docs/reference/rest/v1/projects.locations.keyRings.cryptoKeys.cryptoKeyVersions?hl=en>`__
+            name. This field is available only when the resource's proto
+            contains it.
+
+            To search against the ``kms_key``:
+
+            -  use a field query. Example: ``kmsKey:key``
+            -  use a free text query. Example: ``key``
+        create_time (google.protobuf.timestamp_pb2.Timestamp):
+            The create timestamp of this resource, at which the resource
+            was created. The granularity is in seconds. Timestamp.nanos
+            will always be 0. This field is available only when the
+            resource's proto contains it.
+
+            To search against ``create_time``:
+
+            -  use a field query.
+
+               -  value in seconds since unix epoch. Example:
+                  ``createTime > 1609459200``
+               -  value in date string. Example:
+                  ``createTime > 2021-01-01``
+               -  value in date-time string (must be quoted). Example:
+                  ``createTime > "2021-01-01T00:00:00"``
+        update_time (google.protobuf.timestamp_pb2.Timestamp):
+            The last update timestamp of this resource, at which the
+            resource was last modified or deleted. The granularity is in
+            seconds. Timestamp.nanos will always be 0. This field is
+            available only when the resource's proto contains it.
+
+            To search against ``update_time``:
+
+            -  use a field query.
+
+               -  value in seconds since unix epoch. Example:
+                  ``updateTime < 1609459200``
+               -  value in date string. Example:
+                  ``updateTime < 2021-01-01``
+               -  value in date-time string (must be quoted). Example:
+                  ``updateTime < "2021-01-01T00:00:00"``
+        state (str):
+            The state of this resource. Different resources types have
+            different state definitions that are mapped from various
+            fields of different resource types. This field is available
+            only when the resource's proto contains it.
+
+            Example: If the resource is an instance provided by Compute
+            Engine, its state will include PROVISIONING, STAGING,
+            RUNNING, STOPPING, SUSPENDING, SUSPENDED, REPAIRING, and
+            TERMINATED. See ``status`` definition in `API
+            Reference <https://cloud.google.com/compute/docs/reference/rest/v1/instances>`__.
+            If the resource is a project provided by Cloud Resource
+            Manager, its state will include LIFECYCLE_STATE_UNSPECIFIED,
+            ACTIVE, DELETE_REQUESTED and DELETE_IN_PROGRESS. See
+            ``lifecycleState`` definition in `API
+            Reference <https://cloud.google.com/resource-manager/reference/rest/v1/projects>`__.
+
+            To search against the ``state``:
+
+            -  use a field query. Example: ``state:RUNNING``
+            -  use a free text query. Example: ``RUNNING``
+        additional_attributes (google.protobuf.struct_pb2.Struct):
             The additional searchable attributes of this resource. The
             attributes may vary from one resource type to another.
             Examples: ``projectId`` for Project, ``dnsName`` for DNS
@@ -418,7 +444,7 @@ class ResourceSearchResult(proto.Message):
             provided by the corresponding GCP service (e.g., Compute
             Engine). see `API references and supported searchable
             attributes <https://cloud.google.com/asset-inventory/docs/supported-asset-types#searchable_asset_types>`__
-            for more information.
+            to see which fields are included.
 
             You can search values of these fields through free text
             search. However, you should not consume the field
@@ -431,29 +457,48 @@ class ResourceSearchResult(proto.Message):
                Example: to search
                ``additional_attributes = { dnsName: "foobar" }``, you
                can issue a query ``foobar``.
+        parent_full_resource_name (str):
+            The full resource name of this resource's parent, if it has
+            one. To search against the ``parent_full_resource_name``:
+
+            -  use a field query. Example:
+               ``parentFullResourceName:"project-name"``
+            -  use a free text query. Example: ``project-name``
+        parent_asset_type (str):
+            The type of this resource's immediate parent, if there is
+            one.
+
+            To search against the ``parent_asset_type``:
+
+            -  use a field query. Example:
+               ``parentAssetType:"cloudresourcemanager.googleapis.com/Project"``
+            -  use a free text query. Example:
+               ``cloudresourcemanager.googleapis.com/Project``
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
-    asset_type = proto.Field(proto.STRING, number=2)
-
-    project = proto.Field(proto.STRING, number=3)
-
-    display_name = proto.Field(proto.STRING, number=4)
-
-    description = proto.Field(proto.STRING, number=5)
-
-    location = proto.Field(proto.STRING, number=6)
-
-    labels = proto.MapField(proto.STRING, proto.STRING, number=7)
-
-    network_tags = proto.RepeatedField(proto.STRING, number=8)
-
-    additional_attributes = proto.Field(
-        proto.MESSAGE,
-        number=9,
-        message=struct.Struct,
+    name = proto.Field(proto.STRING, number=1,)
+    asset_type = proto.Field(proto.STRING, number=2,)
+    project = proto.Field(proto.STRING, number=3,)
+    folders = proto.RepeatedField(proto.STRING, number=17,)
+    organization = proto.Field(proto.STRING, number=18,)
+    display_name = proto.Field(proto.STRING, number=4,)
+    description = proto.Field(proto.STRING, number=5,)
+    location = proto.Field(proto.STRING, number=6,)
+    labels = proto.MapField(proto.STRING, proto.STRING, number=7,)
+    network_tags = proto.RepeatedField(proto.STRING, number=8,)
+    kms_key = proto.Field(proto.STRING, number=10,)
+    create_time = proto.Field(
+        proto.MESSAGE, number=11, message=timestamp_pb2.Timestamp,
     )
+    update_time = proto.Field(
+        proto.MESSAGE, number=12, message=timestamp_pb2.Timestamp,
+    )
+    state = proto.Field(proto.STRING, number=13,)
+    additional_attributes = proto.Field(
+        proto.MESSAGE, number=9, message=struct_pb2.Struct,
+    )
+    parent_full_resource_name = proto.Field(proto.STRING, number=19,)
+    parent_asset_type = proto.Field(proto.STRING, number=103,)
 
 
 class IamPolicySearchResult(proto.Message):
@@ -479,13 +524,13 @@ class IamPolicySearchResult(proto.Message):
             set on a resource (like VM instance, Cloud Storage bucket),
             the project field will indicate the project that contains
             the resource. If an IAM policy is set on a folder or
-            orgnization, the project field will be empty.
+            orgnization, this field will be empty.
 
             To search against the ``project``:
 
             -  specify the ``scope`` field as this project in your
                search request.
-        policy (~.gi_policy.Policy):
+        policy (google.iam.v1.policy_pb2.Policy):
             The IAM policy directly set on the given resource. Note that
             the original IAM policy can contain multiple bindings. This
             only contains the bindings that match the given query. For
@@ -503,7 +548,7 @@ class IamPolicySearchResult(proto.Message):
                -  query by the policy contained roles' included
                   permissions. Example:
                   ``policy.role.permissions:compute.instances.create``
-        explanation (~.assets.IamPolicySearchResult.Explanation):
+        explanation (google.cloud.asset_v1.types.IamPolicySearchResult.Explanation):
             Explanation about the IAM policy search
             result. It contains additional information to
             explain why the search result matches the query.
@@ -511,9 +556,8 @@ class IamPolicySearchResult(proto.Message):
 
     class Explanation(proto.Message):
         r"""Explanation about the IAM policy search result.
-
         Attributes:
-            matched_permissions (Sequence[~.assets.IamPolicySearchResult.Explanation.MatchedPermissionsEntry]):
+            matched_permissions (Sequence[google.cloud.asset_v1.types.IamPolicySearchResult.Explanation.MatchedPermissionsEntry]):
                 The map from roles to their included permissions that match
                 the permission query (i.e., a query containing
                 ``policy.role.permissions:``). Example: if query
@@ -528,14 +572,13 @@ class IamPolicySearchResult(proto.Message):
 
         class Permissions(proto.Message):
             r"""IAM permissions
-
             Attributes:
                 permissions (Sequence[str]):
                     A list of permissions. A sample permission string:
                     ``compute.disk.get``.
             """
 
-            permissions = proto.RepeatedField(proto.STRING, number=1)
+            permissions = proto.RepeatedField(proto.STRING, number=1,)
 
         matched_permissions = proto.MapField(
             proto.STRING,
@@ -544,21 +587,10 @@ class IamPolicySearchResult(proto.Message):
             message="IamPolicySearchResult.Explanation.Permissions",
         )
 
-    resource = proto.Field(proto.STRING, number=1)
-
-    project = proto.Field(proto.STRING, number=2)
-
-    policy = proto.Field(
-        proto.MESSAGE,
-        number=3,
-        message=gi_policy.Policy,
-    )
-
-    explanation = proto.Field(
-        proto.MESSAGE,
-        number=4,
-        message=Explanation,
-    )
+    resource = proto.Field(proto.STRING, number=1,)
+    project = proto.Field(proto.STRING, number=2,)
+    policy = proto.Field(proto.MESSAGE, number=3, message=policy_pb2.Policy,)
+    explanation = proto.Field(proto.MESSAGE, number=4, message=Explanation,)
 
 
 class IamPolicyAnalysisState(proto.Message):
@@ -566,7 +598,7 @@ class IamPolicyAnalysisState(proto.Message):
     such as a resource, an identity or an access.
 
     Attributes:
-        code (~.gr_code.Code):
+        code (google.rpc.code_pb2.Code):
             The Google standard error code that best describes the
             state. For example:
 
@@ -581,13 +613,25 @@ class IamPolicyAnalysisState(proto.Message):
             of failure.
     """
 
-    code = proto.Field(
-        proto.ENUM,
-        number=1,
-        enum=gr_code.Code,
-    )
+    code = proto.Field(proto.ENUM, number=1, enum=code_pb2.Code,)
+    cause = proto.Field(proto.STRING, number=2,)
 
-    cause = proto.Field(proto.STRING, number=2)
+
+class ConditionEvaluation(proto.Message):
+    r"""The Condition evaluation.
+    Attributes:
+        evaluation_value (google.cloud.asset_v1.types.ConditionEvaluation.EvaluationValue):
+            The evaluation result.
+    """
+
+    class EvaluationValue(proto.Enum):
+        r"""Value of this expression."""
+        EVALUATION_VALUE_UNSPECIFIED = 0
+        TRUE = 1
+        FALSE = 2
+        CONDITIONAL = 3
+
+    evaluation_value = proto.Field(proto.ENUM, number=1, enum=EvaluationValue,)
 
 
 class IamPolicyAnalysisResult(proto.Message):
@@ -601,14 +645,14 @@ class IamPolicyAnalysisResult(proto.Message):
             of the resource to which the
             [iam_binding][google.cloud.asset.v1.IamPolicyAnalysisResult.iam_binding]
             policy attaches.
-        iam_binding (~.gi_policy.Binding):
+        iam_binding (google.iam.v1.policy_pb2.Binding):
             The Cloud IAM policy binding under analysis.
-        access_control_lists (Sequence[~.assets.IamPolicyAnalysisResult.AccessControlList]):
+        access_control_lists (Sequence[google.cloud.asset_v1.types.IamPolicyAnalysisResult.AccessControlList]):
             The access control lists derived from the
             [iam_binding][google.cloud.asset.v1.IamPolicyAnalysisResult.iam_binding]
             that match or potentially match resource and access
             selectors specified in the request.
-        identity_list (~.assets.IamPolicyAnalysisResult.IdentityList):
+        identity_list (google.cloud.asset_v1.types.IamPolicyAnalysisResult.IdentityList):
             The identity list derived from members of the
             [iam_binding][google.cloud.asset.v1.IamPolicyAnalysisResult.iam_binding]
             that match or potentially match identity selector specified
@@ -621,48 +665,38 @@ class IamPolicyAnalysisResult(proto.Message):
 
     class Resource(proto.Message):
         r"""A Google Cloud resource under analysis.
-
         Attributes:
             full_resource_name (str):
                 The `full resource
                 name <https://cloud.google.com/asset-inventory/docs/resource-name-format>`__
-            analysis_state (~.assets.IamPolicyAnalysisState):
+            analysis_state (google.cloud.asset_v1.types.IamPolicyAnalysisState):
                 The analysis state of this resource.
         """
 
-        full_resource_name = proto.Field(proto.STRING, number=1)
-
+        full_resource_name = proto.Field(proto.STRING, number=1,)
         analysis_state = proto.Field(
-            proto.MESSAGE,
-            number=2,
-            message="IamPolicyAnalysisState",
+            proto.MESSAGE, number=2, message="IamPolicyAnalysisState",
         )
 
     class Access(proto.Message):
         r"""An IAM role or permission under analysis.
-
         Attributes:
             role (str):
                 The role.
             permission (str):
                 The permission.
-            analysis_state (~.assets.IamPolicyAnalysisState):
+            analysis_state (google.cloud.asset_v1.types.IamPolicyAnalysisState):
                 The analysis state of this access.
         """
 
-        role = proto.Field(proto.STRING, number=1, oneof="oneof_access")
-
-        permission = proto.Field(proto.STRING, number=2, oneof="oneof_access")
-
+        role = proto.Field(proto.STRING, number=1, oneof="oneof_access",)
+        permission = proto.Field(proto.STRING, number=2, oneof="oneof_access",)
         analysis_state = proto.Field(
-            proto.MESSAGE,
-            number=3,
-            message="IamPolicyAnalysisState",
+            proto.MESSAGE, number=3, message="IamPolicyAnalysisState",
         )
 
     class Identity(proto.Message):
         r"""An identity under analysis.
-
         Attributes:
             name (str):
                 The identity name in any form of members appear in `IAM
@@ -677,21 +711,17 @@ class IamPolicyAnalysisResult(proto.Message):
                 -  domain:google.com
                 -  allUsers
                 -  etc.
-            analysis_state (~.assets.IamPolicyAnalysisState):
+            analysis_state (google.cloud.asset_v1.types.IamPolicyAnalysisState):
                 The analysis state of this identity.
         """
 
-        name = proto.Field(proto.STRING, number=1)
-
+        name = proto.Field(proto.STRING, number=1,)
         analysis_state = proto.Field(
-            proto.MESSAGE,
-            number=2,
-            message="IamPolicyAnalysisState",
+            proto.MESSAGE, number=2, message="IamPolicyAnalysisState",
         )
 
     class Edge(proto.Message):
         r"""A directional edge.
-
         Attributes:
             source_node (str):
                 The source node of the edge. For example, it
@@ -703,9 +733,8 @@ class IamPolicyAnalysisResult(proto.Message):
                 node or an email of an identity.
         """
 
-        source_node = proto.Field(proto.STRING, number=1)
-
-        target_node = proto.Field(proto.STRING, number=2)
+        source_node = proto.Field(proto.STRING, number=1,)
+        target_node = proto.Field(proto.STRING, number=2,)
 
     class AccessControlList(proto.Message):
         r"""An access control list, derived from the above IAM policy binding,
@@ -728,19 +757,19 @@ class IamPolicyAnalysisResult(proto.Message):
         -  AccessControlList 2: [R2, R3], [P3]
 
         Attributes:
-            resources (Sequence[~.assets.IamPolicyAnalysisResult.Resource]):
+            resources (Sequence[google.cloud.asset_v1.types.IamPolicyAnalysisResult.Resource]):
                 The resources that match one of the following conditions:
 
                 -  The resource_selector, if it is specified in request;
                 -  Otherwise, resources reachable from the policy attached
                    resource.
-            accesses (Sequence[~.assets.IamPolicyAnalysisResult.Access]):
+            accesses (Sequence[google.cloud.asset_v1.types.IamPolicyAnalysisResult.Access]):
                 The accesses that match one of the following conditions:
 
                 -  The access_selector, if it is specified in request;
                 -  Otherwise, access specifiers reachable from the policy
                    binding's role.
-            resource_edges (Sequence[~.assets.IamPolicyAnalysisResult.Edge]):
+            resource_edges (Sequence[google.cloud.asset_v1.types.IamPolicyAnalysisResult.Edge]):
                 Resource edges of the graph starting from the policy
                 attached resource to any descendant resources. The
                 [Edge.source_node][google.cloud.asset.v1.IamPolicyAnalysisResult.Edge.source_node]
@@ -749,38 +778,36 @@ class IamPolicyAnalysisResult(proto.Message):
                 contains the full resource name of a child resource. This
                 field is present only if the output_resource_edges option is
                 enabled in request.
+            condition_evaluation (google.cloud.asset_v1.types.ConditionEvaluation):
+                Condition evaluation for this
+                AccessControlList, if there is a condition
+                defined in the above IAM policy binding.
         """
 
         resources = proto.RepeatedField(
-            proto.MESSAGE,
-            number=1,
-            message="IamPolicyAnalysisResult.Resource",
+            proto.MESSAGE, number=1, message="IamPolicyAnalysisResult.Resource",
         )
-
         accesses = proto.RepeatedField(
-            proto.MESSAGE,
-            number=2,
-            message="IamPolicyAnalysisResult.Access",
+            proto.MESSAGE, number=2, message="IamPolicyAnalysisResult.Access",
         )
-
         resource_edges = proto.RepeatedField(
-            proto.MESSAGE,
-            number=3,
-            message="IamPolicyAnalysisResult.Edge",
+            proto.MESSAGE, number=3, message="IamPolicyAnalysisResult.Edge",
+        )
+        condition_evaluation = proto.Field(
+            proto.MESSAGE, number=4, message="ConditionEvaluation",
         )
 
     class IdentityList(proto.Message):
         r"""The identities and group edges.
-
         Attributes:
-            identities (Sequence[~.assets.IamPolicyAnalysisResult.Identity]):
+            identities (Sequence[google.cloud.asset_v1.types.IamPolicyAnalysisResult.Identity]):
                 Only the identities that match one of the following
                 conditions will be presented:
 
                 -  The identity_selector, if it is specified in request;
                 -  Otherwise, identities reachable from the policy binding's
                    members.
-            group_edges (Sequence[~.assets.IamPolicyAnalysisResult.Edge]):
+            group_edges (Sequence[google.cloud.asset_v1.types.IamPolicyAnalysisResult.Edge]):
                 Group identity edges of the graph starting from the
                 binding's group members to any node of the
                 [identities][google.cloud.asset.v1.IamPolicyAnalysisResult.IdentityList.identities].
@@ -795,38 +822,19 @@ class IamPolicyAnalysisResult(proto.Message):
         """
 
         identities = proto.RepeatedField(
-            proto.MESSAGE,
-            number=1,
-            message="IamPolicyAnalysisResult.Identity",
+            proto.MESSAGE, number=1, message="IamPolicyAnalysisResult.Identity",
         )
-
         group_edges = proto.RepeatedField(
-            proto.MESSAGE,
-            number=2,
-            message="IamPolicyAnalysisResult.Edge",
+            proto.MESSAGE, number=2, message="IamPolicyAnalysisResult.Edge",
         )
 
-    attached_resource_full_name = proto.Field(proto.STRING, number=1)
-
-    iam_binding = proto.Field(
-        proto.MESSAGE,
-        number=2,
-        message=gi_policy.Binding,
-    )
-
+    attached_resource_full_name = proto.Field(proto.STRING, number=1,)
+    iam_binding = proto.Field(proto.MESSAGE, number=2, message=policy_pb2.Binding,)
     access_control_lists = proto.RepeatedField(
-        proto.MESSAGE,
-        number=3,
-        message=AccessControlList,
+        proto.MESSAGE, number=3, message=AccessControlList,
     )
-
-    identity_list = proto.Field(
-        proto.MESSAGE,
-        number=4,
-        message=IdentityList,
-    )
-
-    fully_explored = proto.Field(proto.BOOL, number=5)
+    identity_list = proto.Field(proto.MESSAGE, number=4, message=IdentityList,)
+    fully_explored = proto.Field(proto.BOOL, number=5,)
 
 
 __all__ = tuple(sorted(__protobuf__.manifest))
